@@ -16,6 +16,7 @@
 
 import argparse
 import logging
+import os
 import shutil
 import socket
 import tempfile
@@ -32,6 +33,13 @@ def run_assistant(args):
 
     tmpgpgdir = tempfile.mkdtemp()
     tmpgpg = gpg.GnuPG(tmpgpgdir)
+
+    # link temporary homedir to existing agent
+    agent_socket = homegpg.agent_socket()
+    if agent_socket:
+        os.symlink(agent_socket, os.path.join(tmpgpgdir, 'S.gpg-agent'))
+
+    # import signing candidates
     tmpgpg.import_keys(args.keyring.read(), minimal=True)
 
     window = ui.SigningAssistant(homegpg, tmpgpg)
