@@ -1,5 +1,5 @@
 # This file is part of gcaff
-# Copyright (C) 2013 Fraser Tweedale
+# Copyright (C) 2013-2014  Fraser Tweedale
 #
 # gcaff is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -69,9 +69,14 @@ ALGORITHM = {
     b'1':  'RSA',
     b'16': 'Elgamal (encrypt only)',
     b'17': 'DSA',
+    b'18': 'ECDH',
+    b'19': 'ECDSA',
     b'20': 'Elgamal (sign and encrypt)',
+    b'22': 'EdDSA',
 }
 
+ECC_ALGORITHMS = {'18', '19', '22'}
+
 
 class Uid(object):
     def __init__(self, validity, data):
@@ -173,7 +178,11 @@ class Key(object):
         return [email] if email else self.emails()
 
     def human_algorithm(self):
-        return ALGORITHM.get(self.algorithm, '<unknown algorithm>')
+        return '{} ({}{})'.format(
+            ALGORITHM.get(self.algorithm, '<unknown algorithm>'),
+            self.length,
+            '-bit' if self.algorithm not in ECC_ALGORITHMS else ''
+        )
 
     def human_fingerprint(self):
         fpr = self.fingerprint
@@ -190,6 +199,16 @@ class GnuPG(object):
     signing keys do not have to be moved around.
 
     """
+    def _probe():
+        for exe in ['gpg2', 'gpg']:
+            try:
+                return exe, subprocess.check_output([exe, '--version'])
+            except OSError as e:
+                exc = e
+        raise e
+
+    exe, version = _probe()
+
     def __init__(self, homedir=None):
         # determine the user's normal GNUPGHOME, where we expect
         # to find signing keys
@@ -202,7 +221,7 @@ class GnuPG(object):
 
     def _popen(self, args):
         args = [
-            'gpg',
+            self.exe,
             '--no-tty',
             '--use-agent',
             '--no-auto-check-trustdb',
@@ -217,6 +236,11 @@ class GnuPG(object):
             stderr=subprocess.PIPE
         )
 
+    def agent_socket(self):
+        """Return path to agent socket if using the standard socket."""
+        path = os.path.join(self.homedir, 'S.gpg-agent')
+        return path if os.path.exists(path) else None
+
     def import_keys(self, data, minimal=False):
         """Import keys from the given data.
 
@@ -300,10 +324,14 @@ class GnuPG(object):
                 if seen:
                     keys.append(Key(**kwargs))
                     kwargs = {'uids': []}
-                _, _, kwargs['length'], kwargs['algorithm'] \
-                    = line.split(':')[:4]
+                fields = line.split(':')
+                kwargs['algorithm'] = fields[3]
+                if kwargs['algorithm'] in ECC_ALGORITHMS:
+                    kwargs['length'] = fields[16]  # ECC curve name
+                else:
+                    kwargs['length'] = fields[2]
                 seen += 1
-            if line.startswith('fpr'):
+            if line.startswith('fpr') and 'fingerprint' not in kwargs:
                 kwargs['fingerprint'] = line.split(':')[9]
             elif line.startswith('uid') or line.startswith('pub'):
                 _, validity, _, _, _, _, _, _, _, data = line.split(':')[:10]
@@ -343,7 +371,9 @@ class GnuPG(object):
                     images[-1] += data[16:]
         return images
 
-    STATE_MINIMIZE, STATE_INIT, STATE_UID, STATE_SIGN, STATE_SAVE = range(5)
+    STATE_MINIMIZE, STATE_INIT, STATE_UID, STATE_SIGN, \
+        STATE_GOODPW, STATE_BADPW, STATE_NOPW, STATE_DONE, \
+        STATE_NOAGENT = range(9)
 
     def sign_key_uid(
         self, signkey, keyid, uid, cert_level=0, digest='SHA256',
@@ -376,26 +406,36 @@ class GnuPG(object):
         while p.poll() is None:
             line = p.stdout.readline().strip()
             logger.debug(line)
-            response = self._get_response(line)
+            response = self._get_response(signkey, line)
             if response is not None:
                 logger.debug(response)
                 p.stdin.write(response + '\n')
             # TODO set a max number of iteration and die if
             # exceeded, so we don't get stuck in loop?
         logger.info('gpg process exited code {}'.format(p.returncode))
-        if p.returncode != 0:
-            raise RuntimeError("Bad passphrase")
+        if self.state == self.STATE_BADPW:
+            raise RuntimeError("Incorrect passphrase")
+        elif self.state == self.STATE_NOPW:
+            raise RuntimeError("Passphase not supplied")
+        elif self.state == self.STATE_NOAGENT:
+            raise RuntimeError("Failed to locate gpg-agent")
+        elif self.state != self.STATE_DONE:
+            raise RuntimeError("Unhandled GnuPG behaviour")
 
-    def _get_response(self, line):
+    def _get_response(self, signkey, line):
         msg = line[9:]  # strip "[GNUPG:] "
         lut = {
             "GOT_IT": None,
             "GET_LINE keyedit.prompt": self._get_line_keyedit_prompt,
             "GET_LINE sign_uid.class": '',  # accept default
             "GET_BOOL sign_uid.okay": 'y',
+            "GET_BOOL keyedit.save.okay": 'n',
+            "BAD_PASSPHRASE {}".format(signkey): self._on_bad_passphrase,
+            "MISSING_PASSPHRASE": self._on_missing_passphrase,
+            "GOOD_PASSPHRASE": self._on_good_passphrase,
+            "GET_HIDDEN passphrase.enter": self._on_passphrase_enter,
             # "USERID_HINT <full user id>"
             # "NEED_PASSPHRASE <long keyid> <longkeyid> 1 0"
-            # "BAD_PASSPHRASE <long keyid>" <-- long keyid; exit code 2
             # "ALREADY_SIGNED <long keyid>" <-- exit code 0
         }
         try:
@@ -415,12 +455,33 @@ class GnuPG(object):
         elif self.state == self.STATE_UID:
             self.state = self.STATE_SIGN
             return 'sign'
-        elif self.state == self.STATE_SIGN:
-            self.state = self.STATE_SAVE
+        elif self.state in {
+            self.STATE_BADPW,
+            self.STATE_NOPW,
+            self.STATE_NOAGENT,
+        }:
+            return 'quit'
+        elif self.state in {self.STATE_SIGN, self.STATE_GOODPW}:
+            self.state = self.STATE_DONE
             return 'save'
         else:
             raise RuntimeError('gpg state violation')
 
+    def _on_bad_passphrase(self):
+        if self.state not in {self.STATE_NOPW, self.STATE_NOAGENT}:
+            self.state = self.STATE_BADPW
+
+    def _on_missing_passphrase(self):
+        if self.state != self.STATE_NOAGENT:
+            self.state = self.STATE_NOPW
+
+    def _on_good_passphrase(self):
+        self.state = self.STATE_GOODPW
+
+    def _on_passphrase_enter(self):
+        self.state = self.STATE_NOAGENT
+        return ''
+
 
 class AgentError(StandardError):
     pass
