@@ -1,5 +1,5 @@
 # This file is part of gcaff
-# Copyright (C) 2013 Fraser Tweedale
+# Copyright (C) 2013-2014 Fraser Tweedale
 #
 # gcaff is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ import contextlib
 import logging
 import multiprocessing
 import os
+import smtplib
 import tempfile
 
 import gobject
@@ -64,7 +65,6 @@ class SigningAssistant(gtk.Assistant):
 
         self.uid_selectors = []
         for key in keys:
-            # TODO skip signing key
             uid_selector = UidSelector(key)
             self.append_page(uid_selector)
             self.set_page_type(uid_selector, gtk.ASSISTANT_PAGE_CONTENT)
@@ -128,14 +128,9 @@ class SigningKeyPage(gtk.VBox):
     def __init__(self, keys):
         super(SigningKeyPage, self).__init__()
 
-        self.model = gtk.ListStore(bool, str, str, str)
+        self.model = gtk.ListStore(bool, str, str)
         for key in keys:
-            self.model.append((
-                False,
-                key.length,
-                key.human_algorithm(),
-                key.keyid,
-            ))
+            self.model.append((False, key.human_algorithm(), key.keyid))
 
         treeview = gtk.TreeView(self.model)
         self.pack_start(treeview)
@@ -147,27 +142,21 @@ class SigningKeyPage(gtk.VBox):
         use_col.set_attributes(toggle_renderer, active=0)
         treeview.append_column(use_col)
 
-        text_renderer = gtk.CellRendererText()
-        length_col = gtk.TreeViewColumn('Length')
-        length_col.pack_start(text_renderer, True)
-        length_col.set_attributes(text_renderer, text=1)
-        treeview.append_column(length_col)
-
         text_renderer = gtk.CellRendererText()
         algo_col = gtk.TreeViewColumn('Algorithm')
         algo_col.pack_start(text_renderer, True)
-        algo_col.set_attributes(text_renderer, text=2)
+        algo_col.set_attributes(text_renderer, text=1)
         treeview.append_column(algo_col)
 
         text_renderer = gtk.CellRendererText()
         keyid_col = gtk.TreeViewColumn('Key ID')
         keyid_col.pack_start(text_renderer, True)
-        keyid_col.set_attributes(text_renderer, text=3)
+        keyid_col.set_attributes(text_renderer, text=2)
         treeview.append_column(keyid_col)
 
     def on_toggle(self, cell, path, model):
         model[path][0] = not model[path][0]
-        signing_keys = [keyid for checked, _, _, keyid in model if checked]
+        signing_keys = [keyid for checked, _, keyid in model if checked]
         self.emit('signing-keys-changed', signing_keys)
 
 
@@ -299,19 +288,16 @@ def sign_and_send(tmpgpg, uids, conn):
             tmpgpg.sign_key_uid(*signargs, minimize=True)
             signature = tmpgpg.export_keys([signargs[1]])
             logger.info('got signature: {}'.format(signature))
-        except:
-            logger.exception('error signing key: {}'.format(signargs))
-            signature = None
+        except RuntimeError as e:
+            conn.send((MSG_ERR_FATAL, "Failed to sign keys", e))
+            return
         signatures.append(signature)
         c += 1
-        conn.send((
-            c / n,
-            (True, None) if signature is not None else (False, 'sig fail')
-        ))
+        conn.send((MSG_PROGRESS, c / n))
 
     sigfile = _write_signatures(signatures)
     logger.warn('wrote signatures to file: {}'.format(sigfile))
-    conn.send((None, sigfile))
+    conn.send((MSG_DONE_SIGNING, sigfile))
 
     def _create_message(sig, mailargs):
         try:
@@ -329,12 +315,16 @@ def sign_and_send(tmpgpg, uids, conn):
     logger.info('failed to create {} emails'.format(len(bad_emails)))
     for e in bad_emails:
         c += 1
-        conn.send((c / n, (False, e)))
+        conn.send((MSG_PROGRESS, c / n))
     logger.info('sending {} emails'.format(len(good_emails)))
-    for i in mail.send_messages(good_emails):
-        c += 1
-        conn.send((c / n, (True, None)))
-    conn.send((1.0, (True, True)))
+    try:
+        for i in mail.send_messages(good_emails):
+            c += 1
+            conn.send((MSG_PROGRESS, c / n))
+    except smtplib.SMTPException as e:
+        conn.send((MSG_ERR_FATAL, "Failed to send email", e))
+    conn.send((MSG_PROGRESS, 1.0))
+    conn.send((MSG_DONE_SENDING,))
 
 
 @contextlib.contextmanager
@@ -352,6 +342,10 @@ def _write_signatures(signatures):
         return name
 
 
+MSG_PROGRESS, MSG_DONE_SIGNING, MSG_DONE_SENDING, \
+    MSG_ERR_FATAL, MSG_ERR_RECOVERABLE = range(5)
+
+
 class ProgressPage(gtk.VBox):
     __gsignals__ = {
         'sign-complete': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (str,)),
@@ -405,19 +399,37 @@ class ProgressPage(gtk.VBox):
 
     def on_check(self, conn):
         while conn.poll():
-            fraction, payload = conn.recv()
-            if fraction is None:
-                # status message
-                logger.info('SIGNING COMPLETE')
-                self.emit('sign-complete', payload)
-            else:
-                # progress message
-                self.progress_bar.set_fraction(fraction)
-                if not payload[0]:  # error
-                    logger.error(repr(payload[1]))
-                if fraction >= 1:
-                    self.emit('send-complete')
-                    return False
+            msg = conn.recv()
+
+            if msg[0] == MSG_PROGRESS:
+                self.progress_bar.set_fraction(msg[1])
+
+            elif msg[0] == MSG_DONE_SIGNING:
+                self.emit('sign-complete', msg[1])
+
+            elif msg[0] == MSG_DONE_SENDING:
+                self.emit('send-complete')
+                return False
+
+            elif msg[0] == MSG_ERR_FATAL:
+                dialog = gtk.MessageDialog(
+                    None,
+                    gtk.DIALOG_MODAL,
+                    gtk.MESSAGE_ERROR,
+                    gtk.BUTTONS_CLOSE
+                )
+                dialog.set_property(
+                    'text',
+                    "{}. The program will now exit.".format(msg[1]))
+                dialog.set_property('secondary-text', repr(msg[2]))
+                dialog.connect('response', gtk.main_quit)
+                dialog.show()
+                return False
+
+            elif msg[0] == MSG_ERR_RECOVERABLE:
+                # TODO accumulate recoverable errors for later reporting
+                pass
+
         return True
 
     def on_sign_complete(self, page, sigfile):
